# Luna Testing

## Luna EKS Testing Examples

A few very basic test examples can be used to validate the functionally and operation of Luna.

## ML Use-Cases /w NVIDIA gpu

By default, Luna will autoscale pods with the label elotl-luna=true. Kindly execute the following command `kubectl apply -f` followed by the provided YAML file. 
You will observe a pod running and completing its execution on a p3 instance equipped with an NVIDIA V100 GPU.

Upon completion of the pod, the corresponding node will be automatically terminated.

```
cat <<EOF | kubectl apply -f -
apiVersion: v1
kind: Pod
metadata:
  name: gpu-test
  annotations:
    "node.elotl.co/instance-gpu-skus": "V100"
  labels:
    elotl-luna: "true"
spec:
  restartPolicy: OnFailure
  containers:
    - name: cuda-vector-add
      image: "registry.k8s.io/cuda-vector-add:v0.1"
      resources:
        limits:
          nvidia.com/gpu: 1
EOF
```

You can monitor the creation of the new pod and node by running the command `watch kubectl get pods,nodes`. The test pod will only be active for a brief period after it starts. The GPU node that was added to support the pod will persist for a few more minutes.

To confirm the presence of a GPU on the node, you can run the `kubectl describe node` command and look for the "nvidia.com/gpu" entry or alternatively, you can run the following command:

```
kubectl get nodes "-o=custom-columns=NAME:.metadata.name,KUBELET STATUS:.status.conditions[3].reason,CREATED:metadata.creationTimestamp,VERSION:.status.nodeInfo.kubeletVersion,NVIDIA GPU(s):.status.allocatable.nvidia\.com/gpu"
```

## General Testing (non-ML)

Luna will attempt to consolidate multiple smaller pods onto a newly deployed node, or in the case of larger pods, it will allocate a dedicated node for the pod, as occurs with the bin-selection packing mode.

You can perform simple testing using busybox or other pods of varying sizes. The following YAML files can be utilized, and the number of pods can be adjusted to observe Luna's dynamic response.


**Small busybox deployment**

Several busybox pods will be co-located within a single node

*busybox*
```
cat <<EOF | kubectl apply -f -
apiVersion: apps/v1
kind: Deployment
metadata:
  name: busybox
spec:
  replicas: 6
  selector:
    matchLabels:
      app: busybox
  template:
    metadata:
      labels:
        app: busybox
        elotl-luna: "true"
    spec:
      containers:
        - name: busybox
          image: busybox
          resources:
            requests:
              cpu: 200m
              memory: 128Mi
            limits:
              cpu: 300m
              memory: 256Mi
          command:
            - sleep
            - "infinity"
EOF
```

**Larger busybox deployment**

A single busybox pod will hit the threshold for bin-selection and be located on it's own node

*busybox-large*
```
cat <<EOF | kubectl apply -f -
apiVersion: apps/v1
kind: Deployment
metadata:
  name: busybox-large
spec:
  replicas: 1
  selector:
    matchLabels:
      app: busybox-large
  template:
    metadata:
      labels:
        app: busybox-large
        elotl-luna: "true"
    spec:
      containers:
        - name: busybox-large
          image: busybox
          resources:
            requests:
              cpu: 4
              memory: 256Mi
            limits:
              cpu: 6
              memory: 512Mi
          command:
            - sleep
            - "infinity"
EOF
```

Check that the pods have started, the -o wide opton will show which does the pods are running on:

```
kubectl get pods -l elotl-luna=true -o wide
```
Sample output

```
NAME                             READY   STATUS    RESTARTS   AGE     IP                NODE                                            NOMINATED NODE   READINESS GATES
busybox-65cb45c86b-h9mz8         1/1     Running   0          3m40s   192.168.179.106   ip-192-168-180-235.us-east-2.compute.internal   <none>           <none>
busybox-65cb45c86b-ttgxz         1/1     Running   0          3m40s   192.168.180.96    ip-192-168-180-235.us-east-2.compute.internal   <none>           <none>
busybox-large-57f654fff8-nf4tj   1/1     Running   0          3m32s   192.168.181.40    ip-192-168-177-117.us-east-2.compute.internal   <none>           <none>
```

Next, we can verify the node information to confirm which instance types were selected and added to the Kubernetes cluster by Luna.

```
kubectl get nodes "-o=custom-columns=NAME:.metadata.name,KUBELET STATUS:.status.conditions[3].reason,CREATED:.metadata.creationTimestamp,VERSION:.status.nodeInfo.kubeletVersion,INSTANCE TYPE:.metadata.labels.node\.kubernetes\.io/instance-type,CPU(S):.status.capacity.cpu,MEMORY:.status.capacity.memory" --sort-by=metadata.creationTimestamp
```
Sample output
```
NAME                                            KUBELET STATUS   CREATED                VERSION               INSTANCE TYPE   CPU(S)   MEMORY
ip-192-168-62-44.us-east-2.compute.internal     KubeletReady     2023-02-08T19:52:13Z   v1.24.9-eks-49d8fe8   t3.xlarge       4        16202984Ki
ip-192-168-180-235.us-east-2.compute.internal   KubeletReady     2023-02-10T16:40:11Z   v1.24.9-eks-49d8fe8   t3a.xlarge      4        16248040Ki
ip-192-168-177-117.us-east-2.compute.internal   KubeletReady     2023-02-10T16:40:26Z   v1.24.9-eks-49d8fe8   t3a.2xlarge     8        32608568Ki
```
